"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const tmp_promise_1 = require("tmp-promise");
const addon_1 = require("../../addon");
const ember_cli_entities_1 = require("../../lib/utils/ember-cli-entities");
const { writeFile } = fs_1.promises;
exports.default = ember_cli_entities_1.command({
    name: 'makeup:schema:generate',
    works: 'insideProject',
    description: "Generates the makeup schema from the project's style files.",
    availableOptions: [
        {
            name: 'dependencies',
            type: Boolean,
            default: false,
            aliases: ['d'],
            description: 'Whether to also parse and include style files of dependencies'
        },
        {
            name: 'environment',
            type: String,
            default: 'development',
            aliases: ['e'],
            description: 'The environment to use while building'
        },
        {
            name: 'output',
            type: String,
            default: './makeup.schema.json',
            aliases: ['o'],
            description: 'The file path relative to the project root to write the schema to'
        }
    ],
    run({ environment, output }) {
        return __awaiter(this, void 0, void 0, function* () {
            const Builder = this.project.require('ember-cli/lib/models/builder');
            const owner = this.project.findAddonByName(addon_1.addonName);
            const schemaOutputFile = path_1.resolve(this.project.root, output);
            yield tmp_promise_1.withDir(({ path: outputPath }) => __awaiter(this, void 0, void 0, function* () {
                const builder = new Builder({
                    outputPath,
                    environment,
                    ui: this.ui,
                    project: this.project
                });
                try {
                    yield builder.build();
                }
                finally {
                    yield builder.cleanup();
                }
            }), { unsafeCleanup: true });
            const { usages } = owner;
            // @TODO: write a proper schema generator
            yield writeFile(schemaOutputFile, JSON.stringify(usages, null, 2));
        });
    }
});
