"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Instruments a `BroccoliPlugin` to trigger callbacks when the build is
 * executed. The passed in `plugin` is patched in place, but also returned for
 * convenience.
 *
 * @param plugin The `BroccoliPlugin` to instrument.
 * @param start Optional function called right before the build starts.
 * @param finish Optional function called right after the build finishes.
 */
function hookBroccoliPlugin(plugin, { start, finish }) {
    const { getCallbackObject } = plugin;
    // eslint-disable-next-line no-param-reassign
    plugin.getCallbackObject = function () {
        const callbackObject = getCallbackObject.call(this);
        return {
            build() {
                if (start)
                    start();
                const returnValue = callbackObject.build();
                if (finish) {
                    if (returnValue && typeof returnValue.then === 'function')
                        return returnValue.then((value) => {
                            finish();
                            return value;
                        });
                    finish();
                }
                return returnValue;
            }
        };
    };
    return plugin;
}
exports.hookBroccoliPlugin = hookBroccoliPlugin;
function collectUsages(reportUsages, makePlugin) {
    let usages = [];
    const plugin = makePlugin((usage) => {
        usages.push(usage);
    });
    return hookBroccoliPlugin(plugin, {
        finish() {
            reportUsages(usages);
            usages = [];
        }
    });
}
exports.collectUsages = collectUsages;
