"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const broccoli_persistent_filter_1 = __importDefault(require("broccoli-persistent-filter"));
const cssesc_1 = __importDefault(require("cssesc"));
const postcss_1 = require("postcss");
const config_key_1 = require("../../lib/config-key");
const flattenedThemeToRule = ({ selector, customPropertyPrefix }, theme) => Object.entries(theme).reduce((builtRule, [key, value]) => builtRule.append(postcss_1.decl({
    prop: config_key_1.serializeConfigKey(`${customPropertyPrefix}${key}`),
    value: String(value)
})), postcss_1.rule({ selector }));
function themeToCSS(theme, { customPropertyPrefix, contextClassNamePrefix }) {
    const { contextless, contextual } = theme;
    const css = postcss_1.root();
    css.append(flattenedThemeToRule({ selector: ':root', customPropertyPrefix }, contextless));
    for (const [contextName, contextualTheme] of Object.entries(contextual)) {
        css.append(flattenedThemeToRule({
            selector: `.${cssesc_1.default(`${contextClassNamePrefix}${contextName}`, {
                isIdentifier: true
            })}`,
            customPropertyPrefix
        }, contextualTheme));
    }
    return css.toString();
}
class BroccoliJSONToCSS extends broccoli_persistent_filter_1.default {
    constructor(inputNode, options) {
        super(inputNode, Object.assign({ extensions: ['json'], targetExtension: 'css' }, options));
        this.options = options;
    }
    processString(contents) {
        const json = JSON.parse(contents);
        return themeToCSS(json, this.options);
    }
}
exports.default = BroccoliJSONToCSS;
