"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const broccoli_plugin_1 = __importDefault(require("broccoli-plugin"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const p_props_1 = __importDefault(require("p-props"));
const readFile = util_1.promisify(fs_1.readFile);
const writeFile = util_1.promisify(fs_1.writeFile);
const ENFORCED_GLOB_OPTIONS = {
    absolute: false,
    objectMode: false,
    onlyDirectories: false,
    onlyFiles: true,
    stats: false,
    unique: true
};
const DEFAULT_OPTIONS = {
    indentation: 2,
    pattern: '**/*.json'
};
class BroccoliMergeJSON extends broccoli_plugin_1.default {
    constructor(inputNodes, options = {}) {
        super(inputNodes, Object.assign(Object.assign({}, options), { persistentOutput: false, needsCache: false }));
        this.options = Object.assign(Object.assign({}, DEFAULT_OPTIONS), options);
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            const jsonNodes = yield Promise.all(this.inputPaths.map(inputPath => this.readInputNode(inputPath)));
            const joined = this.joinNodes(jsonNodes);
            const processed = this.options.postprocess
                ? yield this.options.postprocess(joined)
                : joined;
            yield this.writeOutputFile(processed);
        });
    }
    glob(cwd) {
        return __awaiter(this, void 0, void 0, function* () {
            return fast_glob_1.default(this.options.pattern, Object.assign(Object.assign(Object.assign({}, this.options), ENFORCED_GLOB_OPTIONS), { cwd }));
        });
    }
    readJSONFile(absolutePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const json = JSON.parse(yield readFile(absolutePath, 'utf8'));
            if (!this.options.preprocess)
                return json;
            return this.options.preprocess(json);
        });
    }
    readInputNode(inputPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const filePaths = yield this.glob(inputPath);
            const files = yield p_props_1.default(Object.fromEntries(filePaths.map(filePath => [
                filePath,
                this.readJSONFile(path_1.resolve(inputPath, filePath))
            ])));
            return files;
        });
    }
    joinNodes(nodes) {
        return nodes.reduce((mergedNodes, node) => Object.entries(node).reduce((mergedFiles, [filePath, json]) => this.deepSetWithFilePath(mergedFiles, filePath, json), mergedNodes), {});
    }
    deepSetWithFilePath(merged, filePath, value) {
        const keyPath = filePath.slice(0, -'.json'.length).split(path_1.sep);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const leafKey = keyPath.pop();
        const immediateParentNode = keyPath.reduce((needle, keySegment) => {
            if (keySegment in needle) {
                if (typeof needle[keySegment] !== 'object' || !needle[keySegment]) {
                    throw new TypeError(`Expected '${keyPath.join('.')}' to resolve to an object. Failed at '${keySegment}'.`);
                }
            }
            else {
                // eslint-disable-next-line no-param-reassign
                needle[keySegment] = {};
            }
            return needle[keySegment];
        }, merged);
        if (leafKey in immediateParentNode) {
            throw new TypeError(`'${[keyPath, leafKey].join('.')}' is already set.`);
        }
        immediateParentNode[leafKey] = value;
        return merged;
    }
    getOutputFileName(_joined) {
        return this.options.outputFileName || 'merged.json';
    }
    serialize(joined) {
        return JSON.stringify(joined, null, this.options.indentation);
    }
    writeOutputFile(joined) {
        return __awaiter(this, void 0, void 0, function* () {
            const outputFileName = this.getOutputFileName(joined);
            const outputPath = path_1.resolve(this.outputPath, outputFileName);
            const content = this.serialize(joined);
            yield writeFile(outputPath, content);
        });
    }
}
exports.BroccoliMergeJSON = BroccoliMergeJSON;
