"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const plugin_1 = __importDefault(require("ember-css-modules/lib/plugin"));
const postcss_1 = require("../postcss");
/**
 * This is a plugin for `ember-css-modules` that adds the following PostCSS
 * plugins to the build, before the files are joined by CSS Modules:
 *
 * - expandComponentShorthandPlugin: Expands `@component` at-rule shortcuts.
 * - composesContextPlugin: Transforms `@context` at-rules to `composes`
 *   declarations.
 *
 * @see https://github.com/salsify/ember-css-modules#plugins
 */
class EmberCSSModulesPlugin extends plugin_1.default {
    constructor(parent, owner) {
        super(parent);
        this.usages = [];
        this.name = 'ember-makeup:css-modules';
        this.owner = owner;
    }
    flushUsages() {
        this.usages = [];
    }
    collectUsage(usage) {
        this.usages.push(usage);
    }
    reportUsages() {
        this.owner.reportUsages(this.name, this.usages);
    }
    buildEnd() {
        this.reportUsages();
        this.flushUsages();
    }
    config(_environment, baseOptions) {
        this.addPostcssPlugin(baseOptions, 'before', postcss_1.expandComponentShorthandPlugin({
            reportUsage: usage => this.collectUsage(usage)
        }), postcss_1.composesContextPlugin(this.owner.makeupOptions));
    }
}
exports.default = EmberCSSModulesPlugin;
