"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
const config_key_1 = require("../../lib/config-key");
const utils_1 = require("./utils");
/**
 * Transforms the `cfg()` function to `var()`.
 */
exports.default = utils_1.pluginWithRequiredOptions('postcss-ember-makeup:cfg-to-var', ({ keyword = 'cfg', customPropertyPrefix, reportUsage, to }) => {
    const needsTransformation = (value) => value.includes(`${keyword}(`);
    if (!to)
        throw new TypeError('Called without `to` file path option.');
    if (!customPropertyPrefix)
        throw new TypeError('Called without key `customPropertyPrefix` option.');
    return root => {
        root.walkDecls(decl => {
            if (!needsTransformation(decl.value))
                return;
            const rule = decl.parent;
            if (!rule || rule.type !== 'rule') {
                throw decl.error(`'${keyword}' must only be used in declarations contained in rules.`, { word: keyword });
            }
            const originalValue = decl.value;
            decl.value = postcss_value_parser_1.default(decl.value)
                .walk(node => {
                if (node.type !== 'function' || node.value !== keyword)
                    return;
                // This also validates the function node and ensures that it only
                // a single parameter. Thus we can safely reuse it to turn it into a
                // `var` function, without accidentally specifying a second
                // parameter (the optional fallback value).
                const keyNode = utils_1.getKeyNodeFromFunctionNode(decl, node, keyword);
                keyNode.type = 'word';
                const key = keyNode.value;
                keyNode.value = config_key_1.serializeConfigKey(`${customPropertyPrefix}${key}`);
                // turns `cfg` into `var`
                node.value = 'var';
                if (reportUsage) {
                    reportUsage({
                        selectors: rule.selectors,
                        property: decl.prop,
                        originalValue,
                        key,
                        path: to
                    });
                }
            }, true)
                .toString();
        });
    };
});
